@@ -322,8 +322,10 @@ gst_nice_src_dispose (GObject *object)
     g_main_context_unref (src->mainctx);
   src->mainctx = NULL;
 
-  if (src->outbufs)
+  if (src->outbufs) {
+    g_queue_foreach (src->outbufs, (GFunc) gst_buffer_unref, NULL);
     g_queue_free (src->outbufs);
+  }
   src->outbufs = NULL;
 
   G_OBJECT_CLASS (gst_nice_src_parent_class)->dispose (object);
@@ -401,12 +403,24 @@ gst_nice_src_change_state (GstElement * element, GstStateChange transition)
 
   switch (transition) {
     case GST_STATE_CHANGE_NULL_TO_READY:
-      if (src->agent == NULL || src->stream_id == 0 || src->component_id == 0)
+      if (src->agent == NULL)
         {
           GST_ERROR_OBJECT (element,
               "Trying to start Nice source without an agent set");
           return GST_STATE_CHANGE_FAILURE;
         }
+      else if (src->stream_id == 0)
+          {
+            GST_ERROR_OBJECT (element,
+                "Trying to start Nice source without a stream set");
+            return GST_STATE_CHANGE_FAILURE;
+          }
+      else if (src->component_id == 0)
+          {
+            GST_ERROR_OBJECT (element,
+                "Trying to start Nice source without a component set");
+            return GST_STATE_CHANGE_FAILURE;
+          }
       else
         {
           nice_agent_attach_recv (src->agent, src->stream_id, src->component_id,
@@ -416,7 +430,15 @@ gst_nice_src_change_state (GstElement * element, GstStateChange transition)
     case GST_STATE_CHANGE_READY_TO_NULL:
       nice_agent_attach_recv (src->agent, src->stream_id, src->component_id,
           src->mainctx, NULL, NULL);
+      GST_OBJECT_LOCK (src);
+      g_queue_foreach (src->outbufs, (GFunc) gst_buffer_unref, NULL);
+      g_queue_clear (src->outbufs);
+      GST_OBJECT_UNLOCK (src);
       break;
+    case GST_STATE_CHANGE_READY_TO_PAUSED:
+    case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
+    case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
+    case GST_STATE_CHANGE_PAUSED_TO_READY:
     default:
       break;
   }
